@@ -15,88 +15,16 @@ use actix_web::{
 };
 use futures_core::Future;
 
-/// Middleware transform for [`from_fn`].
-pub struct MiddlewareFn<F> {
-    mw_fn: Rc<F>,
-}
-
-impl<S, F, R, B> Transform<S, ServiceRequest> for MiddlewareFn<F>
-where
-    S: Service<ServiceRequest, Response = ServiceResponse<B>, Error = Error> + 'static,
-    F: Fn(ServiceRequest, Next<B>) -> R,
-    R: Future<Output = Result<ServiceResponse<B>, Error>> + 'static,
-    B: MessageBody,
-{
-    type Response = ServiceResponse<B>;
-    type Error = Error;
-    type Transform = MiddlewareFnService<F, B>;
-    type InitError = ();
-    type Future = Ready<Result<Self::Transform, Self::InitError>>;
-
-    fn new_transform(&self, service: S) -> Self::Future {
-        ready(Ok(MiddlewareFnService {
-            service: boxed::rc_service(service),
-            mw_fn: Rc::clone(&self.mw_fn),
-            _phantom: PhantomData,
-        }))
-    }
-}
-
-/// Middleware service for [`from_fn`].
-pub struct MiddlewareFnService<F, B> {
-    service: RcService<ServiceRequest, ServiceResponse<B>, Error>,
-    mw_fn: Rc<F>,
-    _phantom: PhantomData<B>,
-}
-
-impl<F, R, B> Service<ServiceRequest> for MiddlewareFnService<F, B>
-where
-    F: Fn(ServiceRequest, Next<B>) -> R,
-    R: Future<Output = Result<ServiceResponse<B>, Error>> + 'static,
-    B: MessageBody,
-{
-    type Response = ServiceResponse<B>;
-    type Error = Error;
-    type Future = BoxFuture<Result<Self::Response, Self::Error>>;
-
-    forward_ready!(service);
-
-    fn call(&self, req: ServiceRequest) -> Self::Future {
-        let fut = (self.mw_fn)(
-            req,
-            Next::<B> {
-                service: Rc::clone(&self.service),
-            },
-        );
-
-        Box::pin(fut)
-    }
-}
-
-/// Wraps the "next" service in the middleware chain.
-pub struct Next<B> {
-    service: RcService<ServiceRequest, ServiceResponse<B>, Error>,
-}
-
-impl<B> Service<ServiceRequest> for Next<B> {
-    type Response = ServiceResponse<B>;
-    type Error = Error;
-    type Future = BoxFuture<Result<Self::Response, Self::Error>>;
-
-    forward_ready!(service);
-
-    fn call(&self, req: ServiceRequest) -> Self::Future {
-        self.service.call(req)
-    }
-}
-
 /// Wraps an async function to be used as a middleware.
 ///
 /// The wrapped function should have the following form:
 /// ```ignore
 /// use actix_web_lab::middleware::Next;
 ///
-/// async fn my_mw(req: ServiceRequest, next: Next<B>) -> Result<ServiceResponse<B>, Error> {
+/// async fn my_mw(
+///     req: ServiceRequest,
+///     next: Next<impl MessageBody>,
+/// ) -> Result<ServiceResponse<impl MessageBody>, Error> {
 ///     // pre-processing
 ///     next.call(req).await
 ///     // post-processing
@@ -125,6 +53,86 @@ pub fn from_fn<F>(mw_fn: F) -> MiddlewareFn<F> {
     }
 }
 
+/// Middleware transform for [`from_fn`].
+pub struct MiddlewareFn<F> {
+    mw_fn: Rc<F>,
+}
+
+impl<S, F, Fut, B, B2> Transform<S, ServiceRequest> for MiddlewareFn<F>
+where
+    S: Service<ServiceRequest, Response = ServiceResponse<B>, Error = Error> + 'static,
+    F: Fn(ServiceRequest, Next<B>) -> Fut + 'static,
+    Fut: Future<Output = Result<ServiceResponse<B2>, Error>>,
+    B2: MessageBody,
+{
+    type Response = ServiceResponse<B2>;
+    type Error = Error;
+    type Transform = MiddlewareFnService<F, B>;
+    type InitError = ();
+    type Future = Ready<Result<Self::Transform, Self::InitError>>;
+
+    fn new_transform(&self, service: S) -> Self::Future {
+        ready(Ok(MiddlewareFnService {
+            service: boxed::rc_service(service),
+            mw_fn: Rc::clone(&self.mw_fn),
+            _phantom: PhantomData,
+        }))
+    }
+}
+
+/// Middleware service for [`from_fn`].
+pub struct MiddlewareFnService<F, B> {
+    service: RcService<ServiceRequest, ServiceResponse<B>, Error>,
+    mw_fn: Rc<F>,
+    _phantom: PhantomData<B>,
+}
+
+impl<F, Fut, B, B2> Service<ServiceRequest> for MiddlewareFnService<F, B>
+where
+    F: Fn(ServiceRequest, Next<B>) -> Fut,
+    Fut: Future<Output = Result<ServiceResponse<B2>, Error>>,
+    B2: MessageBody,
+{
+    type Response = ServiceResponse<B2>;
+    type Error = Error;
+    type Future = Fut;
+
+    forward_ready!(service);
+
+    fn call(&self, req: ServiceRequest) -> Self::Future {
+        (self.mw_fn)(
+            req,
+            Next::<B> {
+                service: Rc::clone(&self.service),
+            },
+        )
+    }
+}
+
+/// Wraps the "next" service in the middleware chain.
+pub struct Next<B> {
+    service: RcService<ServiceRequest, ServiceResponse<B>, Error>,
+}
+
+impl<B> Next<B> {
+    /// Equivalent to `Service::call(self, req)`.
+    pub fn call(&self, req: ServiceRequest) -> <Self as Service<ServiceRequest>>::Future {
+        Service::call(self, req)
+    }
+}
+
+impl<B> Service<ServiceRequest> for Next<B> {
+    type Response = ServiceResponse<B>;
+    type Error = Error;
+    type Future = BoxFuture<Result<Self::Response, Self::Error>>;
+
+    forward_ready!(service);
+
+    fn call(&self, req: ServiceRequest) -> Self::Future {
+        self.service.call(req)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use actix_web::{
@@ -151,8 +159,17 @@ mod tests {
             Ok(res)
         }
 
+        async fn mutate_body_type(
+            req: ServiceRequest,
+            next: Next<impl MessageBody + 'static>,
+        ) -> Result<ServiceResponse<impl MessageBody>, Error> {
+            let res = next.call(req).await?;
+            Ok(res.map_into_left_body::<()>())
+        }
+
         let app = test::init_service(
             App::new()
+                .wrap(from_fn(mutate_body_type))
                 .wrap(from_fn(add_res_header))
                 .wrap(Logger::default())
                 .wrap(from_fn(noop))
@@ -164,4 +181,57 @@ mod tests {
         let res = test::call_service(&app, req).await;
         assert!(res.headers().contains_key(header::WARNING));
     }
+
+    #[actix_web::test]
+    async fn closure_capture_and_return_from_fn() {
+        struct MyMw(bool);
+
+        impl MyMw {
+            async fn mw_cb(
+                &self,
+                req: ServiceRequest,
+                next: Next<impl MessageBody + 'static>,
+            ) -> Result<ServiceResponse<impl MessageBody>, Error> {
+                let mut res = match self.0 {
+                    true => req.into_response("short-circuited").map_into_right_body(),
+                    false => next.call(req).await?.map_into_left_body(),
+                };
+                res.headers_mut()
+                    .insert(header::WARNING, HeaderValue::from_static("42"));
+                Ok(res)
+            }
+
+            pub fn into_middleware<S, B>(
+                self,
+            ) -> impl Transform<
+                S,
+                ServiceRequest,
+                Response = ServiceResponse<impl MessageBody>,
+                Error = Error,
+                InitError = (),
+            >
+            where
+                S: Service<ServiceRequest, Response = ServiceResponse<B>, Error = Error> + 'static,
+                B: MessageBody + 'static,
+            {
+                let this = Rc::new(self);
+                from_fn(move |req, next| {
+                    let this = Rc::clone(&this);
+                    async move { Self::mw_cb(&this, req, next).await }
+                })
+            }
+        }
+
+        let app = test::init_service(
+            App::new()
+                .wrap(Logger::default())
+                .wrap(MyMw(true).into_middleware())
+                .wrap(Logger::default()),
+        )
+        .await;
+
+        let req = test::TestRequest::default().to_request();
+        let res = test::call_service(&app, req).await;
+        assert!(res.headers().contains_key(header::WARNING));
+    }
 }
